"""
Defines basic urls.

django_mailjet_subscription_form
    Url to the basic form view: SubscriptionView.

django_mailjet_subscription_success
    Url to the basic success template.
"""

from django.conf.urls import patterns, url
from django.views import generic

import views

urlpatterns = patterns(
    'newsletter.views',
    url(r'subscription/$', views.SubscriptionView.as_view(),
        name='django_mailjet_subscription_form'),
    url(r'subscription/success/$', generic.TemplateView.as_view(
        template_name='django_mailjet/subscription_success.html'),
        name='django_mailjet_subscription_success'),
)



'''
Introduction
==============================================================================

``mailjet.contrib.django_mailjet`` provides generic tools to plug
basic mailjet support in your project.

Advanced users may use
``mailjet.contrib.django_mailjet.forms.SubscriptionForm`` directly
without installing anything.

Installation
==============================================================================

In ``settings.py``:

 - set ``MAILJET_LIST_NAME`` and ``MAILJET_LIST_LABEL``, note that
   it may not contain non-alphanumeric characters, those are used as
   defaults for ``mailjet.contrib.django_mailjet.forms.SubscriptionForm``,
 - add to ``INSTALLED_APPS``: ``mailjet.contrib.django_mailjet``
   for templates to be loadable.

Include ``mailjet.contrib.django_mailjet.urls`` in ``urls.py``, ie.:

.. code:: py

    url(r'^newsletter/', include('mailjet.contrib.django_mailjet.urls')),

Users may now subscribe to your mailing list on ``/newsletter/``.
'''




from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django import forms

import mailjet


class SubscriptionForm(forms.Form):
    """ Simple subscription form """
    email = forms.EmailField()

    def __init__(self, list_name=None, list_label=None, *args, **kwargs):
        """
        If `list_name` and `list_label` are None, then
        `settings.MAILJET_LIST_NAME` and `settings.MAILJET_LIST_LABEL` will be
        used.
        """
        self.list_name = list_name or settings.MAILJET_LIST_NAME
        self.list_label = list_label or settings.MAILJET_LIST_LABEL

        super(SubscriptionForm, self).__init__(*args, **kwargs)

    def clean_email(self):
        """ Raise ValidationError if the contact exists. """
        contacts = self.api.lists.contacts(id=self.list_id)['result']

        for contact in contacts:
            if contact['email'] == self.cleaned_data['email']:
                raise forms.ValidationError(
                    _(u'This email is already subscribed'))

        return self.cleaned_data['email']

    def save(self):
        """ Call `add_contact()` """
        self.add_contact()

    def add_contact(self):
        """ Create a contact with using the email on the list. """
        self.api.lists.addcontact(
            contact=self.cleaned_data['email'], id=self.list_id, method='POST')

    @property
    def api(self):
        """ Get or create an Api() instance using django settings. """
        api = getattr(self, '_api', None)

        if api is None:
            self._api = mailjet.Api()

        return self._api

    @property
    def list_id(self):
        """ Get or create the list id. """
        list_id = getattr(self, '_list_id', None)

        if list_id is None:
            for l in self.api.lists.all()['lists']:
                if l['name'] == self.list_name:
                    self._list_id = l['id']

            if not getattr(self, '_list_id', None):
                self._list_id = self.api.lists.create(
                    label=self.list_label, name=self.list_name,
                    method='POST')['list_id']

        return self._list_id



from django.views import generic
from django.core.urlresolvers import reverse_lazy

from .forms import SubscriptionForm


class SubscriptionView(generic.FormView):
    """ Basic subscription views """
    form_class = SubscriptionForm
    template_name = 'django_mailjet/subscription_form.html'
    success_url = reverse_lazy('django_mailjet_subscription_success')

    def form_valid(self, form):
        """ Call `form.save()` and super itself. """
        form.save()
        return super(SubscriptionView, self).form_valid(form)



from mailjet.connection import Connection
import json


class Api(object):
    def __init__(self, connection=None, api_key=None, secret_key=None):
        if not connection:
            connection = Connection.get_connection(api_key, secret_key)

        self.connection = connection

    def __getattr__(self, method):
        return ApiMethod(self, method)


class ApiMethod(object):
    def __init__(self, api, method):
        self.api = api
        self.method = method

    def __getattr__(self, function):
        return ApiMethodFunction(self, function)

    def __unicode__(self):
        return self.method


class ApiMethodFunction(object):
    def __init__(self, method, function):
        self.method = method
        self.function = function

    def __call__(self, **kwargs):
        if kwargs.pop('method', 'GET') == 'POST':
            postdata = kwargs
            for key in postdata:
                if isinstance(postdata[key], tuple):
                    for idx, item in enumerate(postdata[key]):
                        postdata['%s[%d]' % (key, idx)] = item
                    del postdata[key]
            options = None
        else:
            options = kwargs
            postdata = None

        response = self.method.api.connection.open(
            self.method,
            self.function,
            options=options,
            postdata=postdata,
        )

        if response.msg == u'No Content':
            return None

        return json.load(response)

    def __unicode__(self):
        return self.function




import os
import setuptools

if os.path.isfile('README.rst'):
    long_description = open('README.rst').read()
else:
    long_description = 'See http://pypi.python.org/pypi/mailjet/'

setuptools.setup(
    name='mailjet',
    version='1.4.1',
    author='Rick van Hattem',
    author_email='Rick.van.Hattem@Fawo.nl',
    description='mailjet is a django app to implement the mailjet REST API',
    url='https://github.com/WoLpH/mailjet',
    license='BSD',
    packages=setuptools.find_packages(),
    long_description=long_description,
    test_suite='nose.collector',
    tests_require=['nose'],
    classifiers=[
        'License :: OSI Approved :: BSD License',
    ],
)




import time

from django.test import TestCase

import mailjet

from .forms import SubscriptionForm


class SubscriptionFormTest(TestCase):
    def list_name_and_label(self):
        ts = int(time.time())

        result = (u'Test%s' % ts, u'test%s' % ts)

        self.temporary_lists.append(result)

        return result

    def setUp(self):
        self.temporary_lists = []
        self.list_name, self.list_label = self.list_name_and_label()

        self.api = mailjet.Api()

    def tearDown(self):
        lists = self.api.lists.all()['lists']

        for temporary_list in self.temporary_lists:
            for l in lists:
                if l['name'] == temporary_list[0]:
                    self.api.lists.delete(method='POST', id=l['id'])

    def test_add_contact(self):
        form = SubscriptionForm(
            data={'email': 'james@example.com'}, list_name=self.list_name,
            list_label=self.list_label)

        self.assertTrue(
            form.is_valid(), 'Form should be valid for test to continue')

        form.add_contact()

        for c in self.api.lists.contacts(id=form.list_id)['result']:
            if c['email'] == 'james@example.com':
                return

        self.fail('Contact was not created')

    def test_settings_override(self):
        name_fixture, label_fixture = self.list_name_and_label()

        with self.settings(MAILJET_LIST_NAME=name_fixture):
            form = SubscriptionForm()
            self.assertEqual(name_fixture, form.list_name)

            form = SubscriptionForm(list_name=self.list_name)
            self.assertEqual(self.list_name, form.list_name)

        # self.settings context manager cannot be used to override 2 settings
        with self.settings(MAILJET_LIST_LABEL=label_fixture):
            form = SubscriptionForm()
            self.assertEqual(label_fixture, form.list_label)

            form = SubscriptionForm(list_label=self.list_label)
            self.assertEqual(self.list_label, form.list_label)

    def test_save(self):
        def contact_in_list(email):
            for c in self.api.lists.contacts(id=form.list_id)['result']:
                if c['email'] == email:
                    return True
            return False

        form = SubscriptionForm(
            data={'email': 'james@example.com'}, list_name=self.list_name,
            list_label=self.list_label)
        self.assertTrue(
            form.is_valid(), 'Form should be valid for test to continue')
        self.assertFalse(
            contact_in_list('james@example.com'),
            'Contact must not be in list for test to continue')

        form.save()
        self.assertTrue(contact_in_list('james@example.com'))

        form = SubscriptionForm(
            data={'email': 'rick@example.com'}, list_name=self.list_name,
            list_label=self.list_label)
        self.assertTrue(
            form.is_valid(), 'Form should be valid for test to continue')
        self.assertFalse(
            contact_in_list('rick@example.com'),
            'Contact must not be in list for test to continue')

        form.save()
        self.assertTrue(contact_in_list('rick@example.com'))

    def test_clean_email(self):
        form = SubscriptionForm(
            data={'email': 'james@example.com'}, list_name=self.list_name,
            list_label=self.list_label)

        self.assertTrue(
            form.is_valid(), 'Form should be valid for test to continue')

        form.save()

        form = SubscriptionForm(
            data={'email': 'james@example.com'}, list_name=self.list_name,
            list_label=self.list_label)

        self.assertFalse(
            form.is_valid(), 'Form should detect duplicate email')
